#include "TRAEMain.h"
#include "TRAEDetector.h"
#include "TRAEParticle.h"
#include "TRAEEvent.h"
#include "TRandom.h"
#include "TFile.h"
#include "TTree.h"
#include "Riostream.h"

ClassImp(TRAEMain)

TRAEMain::TRAEMain()
{
	DetectorList = new TList;
	DetectorList->SetOwner(kTRUE);
	Event = new TRAEEvent;
}

TRAEMain::~TRAEMain()
{
	DetectorList->Clear();
	delete DetectorList;
	Event->Clear();
	delete Event;
}

void TRAEMain::WriteTree(const Int_t n_events, const Char_t *filename)
{
	// Create list of detectors, write in TFile filename
	// Create and fill TTree with n_events, write in TFile filename
	
	// Open new ROOT file
	TFile *RootFile = new TFile(filename, "RECREATE");
	
	// Create TTree for events
	TTree *RootTree = new TTree("Run", "TTree with event objects");
	RootTree->SetAutoSave(30000000);
	
	// Create branch for events
	RootTree->Branch("Event", "TRAEEvent", &Event, 64000, 0)->SetAutoDelete(kFALSE);
	
	// Fill list of detectors
	for(Int_t i=1; i<=5; i++){
		TRAEDetector *Det = new TRAEDetector( Form("Det_%d",i), "TRAEDetector object" );
		DetectorList->Add( Det );
	}
	// Write detectors in ROOT file
	DetectorList->Write();
	
	// Loop creating events and filling tree
	for(Int_t i=1; i<=n_events; i++){
		FillEvent(i);
		cout << " ### EVENT NUMBER " << i << " ###" << endl;
		Event->Print();
		RootTree->Fill();
		Event->Clear();
	}
	
	// Write TTree in file
	RootTree->Write();
	
	// Delete detectors
	DetectorList->Clear();
	
	// Close file
	RootFile->Close();
}

void TRAEMain::FillEvent(Int_t ev_no)
{
	// Generate random event :
	//   - random number of particles (0 < n < 11)
	//   - each particle associated to random number (0 < n < 6) of detectors
	
	static Int_t MultMax = 10;
	static Int_t NDetMax = 5;
	
	Int_t mult = 1 + gRandom->Integer(MultMax);
	
	for(Int_t ip = 1; ip <= mult; ip++){
		
		// new particle
		TRAEParticle *Particle = Event->AddParticle();
		Particle->SetName( Form("Part_%d_Event_%d", ip, ev_no) );
		
		// associate to detectors
		Int_t ndet = 1 + gRandom->Integer(NDetMax);
		for(Int_t id = 0; id < ndet; id++){
			Particle->AddDetector( (TRAEDetector*)DetectorList->At(id) );
		}
	}
}
