#include "TRAEParticle.h"
#include "TRAEDetector.h"
#include "Riostream.h"
#include "TBuffer.h"

ClassImp(TRAEParticle)

#define MAX_NUM_DET 5

TRAEParticle::TRAEParticle()
	: fDetectors(MAX_NUM_DET)
{
   //default ctor.
   //TRefArray is initialised with size MAX_NUM_DET
	fNumDet = 0;
}

TRAEParticle::~TRAEParticle()
{
	fDetectors.Delete();
}

void TRAEParticle::AddDetector(TRAEDetector* det)
{
	// add detector to list of detectors hit by particle
	
   if (fNumDet == MAX_NUM_DET) {
      Warning("AddDetector",
              "Cannot add more than %d detectors. Detector not taken into account",
              fNumDet);
      return;
   }
   fDetectors.AddAt(det, fNumDet);//this will not expand the array if fNumDet is bigger than MAX_NUM_DET
	fNumDet++;
}

void TRAEParticle::Clear(Option_t * opt)
{
   // Clear list of detectors

   fDetectors.Delete();
	fNumDet = 0;
}

void TRAEParticle::Print(Option_t * option) const
{
	cout << "TRAEParticle::" << GetName() << " fNumDet=" << fNumDet << endl;
   if (fNumDet) {
		for(int i=0; i<fNumDet; i++){
			TObject *d = fDetectors.At(i);
			if(d) d->Print();
		}
   }
}

//______________________________________________________________________________
void TRAEParticle::Streamer(TBuffer &R__b)
{
   // Stream an object of class TRAEParticle.

	Info("Streamer","Called");
   UInt_t R__s, R__c;
   if (R__b.IsReading()) {
      Version_t R__v = R__b.ReadVersion(&R__s, &R__c); if (R__v) { }
      TNamed::Streamer(R__b);
      R__b >> fNumDet;
		printf("Read fNumDet=%d\n",fNumDet);
      fDetectors.Streamer(R__b);
		fDetectors.Print();
      R__b.CheckByteCount(R__s, R__c, TRAEParticle::IsA());
   } else {
      R__c = R__b.WriteVersion(TRAEParticle::IsA(), kTRUE);
      TNamed::Streamer(R__b);
      R__b << fNumDet;
      fDetectors.Streamer(R__b);
      R__b.SetByteCount(R__c, kTRUE);
   }
}

